


; Generates ions randomly on the filament
; with random el and az angles
; Appelhans  November 2002


; definition of user adjustable variables  -----------------------

			 ; ion flight control parameters

defa _rerun 					0		; to turn rerun on (1) or off (0)

  ; ---------- adjustable at beginning of flight -----------------

			 ; ion randomization parameters

defa _el_Angle_Off_Vel_Axis   90.0	  ; (+- 90 deg) cone angle
defa _az_Angle_Off_Vel_Axis   90.0	  ; (+- 90 deg) cone angle
defa _x_Random_Offset_mm	   0.50 	; random offset for x
defa _y_Random_Offset_mm	   9.0	   ; random offset for y
defa _z_Random_Offset_mm	   0.0	   ; random offset for z


; definition of static variables -----------------------------
									  ; trap flags and process parameters
defs first					   0.0	  ; first call flag
defs initial_energy 		   0.0	  ; initial energy of current ion
defs initial_energy_flag	   0.0	  ; compute initial energy of ion if 0


; program segments below --------------------------------------------

seg initialize		; randomize ion's position, and direction

			rcl _rerun
			sto rerun_flym				; turn rerun mode on or off
   ;	 1 sto Rerun_Flym				; force rerun on
										; turns traj file saving off
		;------------------- get ion's initial velocity components -------------
		rcl ion_vz_mm					; get ion's specified velocity components
		rcl ion_vy_mm
		rcl ion_vx_mm

		;------------------- convert to 3d polar coords -------------
		>p3d							; convert to polar 3d

		;------------------- save polar coord values ----------------
		sto speed rlup					; store ion's speed
		sto az_angle rlup				; store ion's az angle
		sto el_angle					; store ion's el angle


		;------------------- make sure el_Angle_Off_Vel_Axis is legal -------------
								; force 0 <= el_Angle_Off_Vel_Axis <= 180
		rcl _el_Angle_Off_Vel_Axis abs
		180 x>y rlup sto _el_Angle_Off_Vel_Axis


		;-- compute randomized el angle

		2 rcl _el_Angle_Off_Vel_Axis * rand *
		rcl _el_Angle_Off_Vel_Axis - ;

		;-- compute randomized az angle

		2 rcl _az_Angle_Off_Vel_Axis * rand *
		rcl _az_Angle_Off_Vel_Axis - 90 +


		;---------------------- recall new ion speed ------------------
		rcl speed						; recall new speed

		;--------- at this point x = speed, y = az, z = el --------------
		;------------- convert to rectangular velocity components ---------
		>r3d							; convert polar 3d to rect 3d

		;------------- el rotate back to from 90 vertical -------------
		-90 >elr

		;------------- el rotate back to starting elevation -------------
		rcl el_angle >elr

		;------------- az rotate back to starting azimuth -------------
		rcl az_angle >azr

		;------------- update ion's velocity components with new values --------
		sto ion_vx_mm					; return vx
		rlup
		sto ion_vy_mm					; return vy
		rlup
		sto ion_vz_mm					; return vz

		;--------- randomize ion's position components --------
										   ; x component
		rcl _x_Random_Offset_mm
		2 / sto x_half_pos				   ; save half max shift

		rcl ion_px_mm					   ; get nominal x start
		rcl _x_Random_Offset_mm rand * +   ; add random shift
		rcl x_half_pos -				   ; subtract half shift
		sto ion_px_mm					   ; store random x start

										   ; y component
		rcl _y_Random_Offset_mm
		2 / sto y_half_pos				   ; save half max shift

		rcl ion_py_mm					   ; get nominal y start
		rcl _y_Random_Offset_mm rand * +   ; add random shift
		rcl y_half_pos -				   ; subtract half shift
		sto ion_py_mm					   ; store random y start

										   ; z component

		rcl _z_Random_Offset_mm
		2 / sto z_half_pos				   ; save half max shift

		rcl ion_pz_mm					   ; get nominal z start
		rcl _z_Random_Offset_mm rand * +   ; add random shift
		rcl z_half_pos -				   ; subtract half shift
		sto ion_pz_mm					   ; store random z start

		;---------------------- done -----------------------------------------


seg Terminate
;			 rcl _rerun
;			 sto rerun_flym 			   ; reset rerun mode
